﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;

using Brew;

namespace Brew.Webforms.Widgets {

	[TargetControlType(typeof(WebControl))]
	public class Position : Widget {

		public Position() : base("position") {		}

		public override List<WidgetEvent> GetEvents() {
			return new List<WidgetEvent>() { };
		}

		public override List<WidgetOption> GetOptions() {
			return new List<WidgetOption>() {
				new WidgetOption { Name = "my", DefaultValue = null },
				new WidgetOption { Name = "at", DefaultValue = null },
				new WidgetOption { Name = "of", DefaultValue = null },
				new WidgetOption { Name = "collision", DefaultValue = null },
				new WidgetOption { Name = "using", DefaultValue = null }
			};
		}

		protected override void OnLoad(EventArgs e) {
			base.OnLoad(e);

			if (Of != null && FindControl(Of) != null) {
				Of = "#" + FindControl(Of).ClientID;
			}
		}

		#region .    Options    .

		/// <summary>
		/// Defines which position on the element being positioned to align with the target element: "horizontal vertical" alignment. A single value such as "right" will default to "right center", "top" will default to "center top" (following CSS convention). Acceptable values: "top", "center", "bottom", "left", "right". Example: "left top" or "center center"
		/// Reference: http://api.jqueryui.com/position/#option-my
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(null)]
		[Description("Defines which position on the element being positioned to align with the target element: \"horizontal vertical\" alignment. A single value such as \"right\" will default to \"right center\", \"top\" will default to \"center top\" (following CSS convention). Acceptable values: \"top\", \"center\", \"bottom\", \"left\", \"right\". Example: \"left top\" or \"center center\"")]
		public string My { get; set; }

		/// <summary>
		/// Defines which position on the target element to align the positioned element against: "horizontal vertical" alignment. A single value such as "right" will default to "right center", "top" will default to "center top" (following CSS convention). Acceptable values: "top", "center", "bottom", "left", "right". Example: "left top" or "center center"
		/// Reference: http://api.jqueryui.com/position/#option-at
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(null)]
		[Description("Defines which position on the target element to align the positioned element against: \"horizontal vertical\" alignment. A single value such as \"right\" will default to \"right center\", \"top\" will default to \"center top\" (following CSS convention). Acceptable values: \"top\", \"center\", \"bottom\", \"left\", \"right\". Example: \"left top\" or \"center center\"")]
		public string At { get; set; }

		/// <summary>
		/// Element to position against. If you provide a selector, the first matching element will be used. If you provide a jQuery object, the first element will be used. If you provide an event object, the pageX and pageY properties will be used. Example: "#top-menu"
		/// Reference: http://api.jqueryui.com/position/#option-of
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(null)]
		[Description("Element to position against. If you provide a selector, the first matching element will be used. If you provide a jQuery object, the first element will be used. If you provide an event object, the pageX and pageY properties will be used. Example: \"#top-menu\"")]
		public string Of { get; set; }

		[Obsolete("The offset option has been deprecated in jQuery UI 1.9 and offsets have been merged into the my and at options. Use \"at\" and \"my\" options instead.", true)]
		public string Offset { get; set; }

		/// <summary>
		/// When the positioned element overflows the window in some direction, move it to an alternative position. Similar to my and at, this accepts a single value or a pair for horizontal/vertical, eg. "flip", "fit", "fit flip", "fit none".
		/// Reference: http://api.jqueryui.com/position/#option-collision
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(null)]
		[Description("When the positioned element overflows the window in some direction, move it to an alternative position. Similar to my and at, this accepts a single value or a pair for horizontal/vertical, eg. \"flip\", \"fit\", \"fit flip\", \"fit none\".")]
		public string Collision { get; set; }

		/// <summary>
		/// When specified the actual property setting is delegated to this callback. Receives a single parameter which is a hash of top and left values for the position that should be set.
		/// Reference: http://api.jqueryui.com/position/#option-using
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(null)]
		[Description("When specified the actual property setting is delegated to this callback. Receives a single parameter which is a hash of top and left values for the position that should be set.")]
		public string Using { get; set; }

		#endregion
	}
}
